#!/bin/sh

[ -f /etc/profile ] && . /etc/profile >/dev/null

SYSTEM_TYPE=""
LAN_IF_NAME=""
LAN_IF_ADDR=""

FIREWALL_LOG=""

DEFAULT_BRIDGE_VALUE=0
BRIDGE_VALUE=""
MODEL=""
WORK_DIR="/tmp/xunyou"
BASEDIR="$(cd "$(dirname "$0")"; pwd -P)"
XUNYOU_CHAIN="XUNYOU"
XUNYOU_ACC_CHAIN="XUNYOU_ACC"
XUNYOU_PROXY_CHAIN="XUNYOU_PROXY"
XUNYOU_CDN_CHAIN="XUNYOU_CDN"
TABLE_ID="95"
PROXY_PORT=9800
ACC_CONF="${WORK_DIR}/conf/accelerate"


get_json_value()
{
    local json=${1}
    local key=${2}
    local line=`echo ${json} | tr -d "\n " | awk -F"[][}{,]" '{for(i=1;i<=NF;i++) {if($i~/^"'${key}'":/) print $i}}' | tr -d '"' | sed -n 1p`
    local value=${line#*:}
    echo ${value}
}

env_init()
{
    local hostname=`uname -n`

    if [ -d "/koolshare" ]; then
        SYSTEM_TYPE="merlin"
        LAN_IF_NAME="br0"
        MODEL=`nvram get productid`
    elif [ -d "/jffs" ]; then
        SYSTEM_TYPE="asus"
        LAN_IF_NAME="br0"
        MODEL=`nvram get productid`
    elif [ -d "/var/tmp/misc2" ]; then
        SYSTEM_TYPE="linksys"
        LAN_IF_NAME="br0"
        MODEL=`nvram kget modelNumber`
    elif [ -d "/etc/oray" ]; then
        SYSTEM_TYPE="oray"
        LAN_IF_NAME="br-lan"
        MODEL=`awk -F '=' '$1=="DEVICE_REVISION" {print $2}' /etc/device_info  | tr -d \'\"`
    else
        local hostname=`uname -n`
        if [ "${hostname}" == "XiaoQiang" ]; then
            SYSTEM_TYPE="xiaomi"
            LAN_IF_NAME="br-lan"
            MODEL=`uci get /usr/share/xiaoqiang/xiaoqiang_version.version.HARDWARE`
        else
            curl -s --connect-timeout 3 --retry 3 http://127.0.0.1/currentsetting.htm | tr '\r' '\n' > /tmp/.xunyou_tmp
            if [ ! -f /tmp/.xunyou_tmp ]; then
                echo "Failed: curl -s --connect-timeout 3 --retry 3 http://127.0.0.1/currentsetting.htm"
                return 1
            fi

            local model=`awk -F"=" '$1=="Model" {print $2}' /tmp/.xunyou_tmp`
            local version=`awk -F"=" '$1=="Firmware" {print $2}' /tmp/.xunyou_tmp`
            
            rm -f /tmp/.xunyou_tmp

            if [ -n "${model}" -a -n "${version}" ]; then
                SYSTEM_TYPE="netgear"
                LAN_IF_NAME="br0"
                MODEL="${model}"
            elif [ -f /etc/openwrt_release ]; then
                LAN_IF_NAME="br-lan"
                PLUGIN_DIR="/xunyou"
                SYSTEM_TYPE="koolshare"
                VENDOR="KOOLSHARE"
                MODEL="OPENWRT"
                VERSION=`cat /etc/openwrt_version`
            else
                echo "Unknown system type!"
                return 1
            fi
        fi
    fi
    
    local lan_if_name=$(grep "lan_if_name=" "${BASEDIR}/lan.config" 2>/dev/null | head -n1 | cut -d= -f2)
    [ -n "${lan_if_name}" ] && LAN_IF_NAME="${lan_if_name}"
    

    LAN_IF_ADDR=`ip address show ${LAN_IF_NAME} | grep "\<inet\>" | head -n 1 | awk -F ' ' '{print $2}' | awk -F '/' '{print $1}'`
    if [ -z "${LAN_IF_ADDR}" ]; then
        echo "Can't find the lan ip!"
        return 1
    fi

    LAN_IF_MAC=`ip address show ${LAN_IF_NAME} | grep link/ether | awk -F ' ' '{print $2}'`
    if [ -z "${LAN_IF_MAC}" ]; then
        echo "Can't find the lan mac!"
        return 1
    fi
    
    if [ "${SYSTEM_TYPE}" == "xiaomi" ]; then
        if [ ${MODEL} == "RB04" -o ${MODEL} == "RB08" ]; then
            DEFAULT_BRIDGE_VALUE=1
        fi
        
        if [ ${MODEL} == "RB05" -o ${MODEL} == "RB06" -o ${MODEL} == "RA72" ]; then
            DEFAULT_BRIDGE_VALUE=0
        fi
    fi
    return 0
}

check_rule()
{
    echo "begin check_rule">> /tmp/123.txt
    iptables -t mangle -n -L ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ];then
        echo "Mangle chain ${XUNYOU_CHAIN} hasn't been configurated!"
        return 1
    fi
    
    iptables -t mangle -C PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "Mangle rule hasn't been configurated in chain PREROUTING"
        return 1
    fi
    
    iptables -t nat -n -L ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ];then
        echo "Nat chain ${XUNYOU_CHAIN} hasn't been configurated!"
        return 1
    fi
    
    iptables -t nat -C PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        echo "Nat rule hasn't been configurated in chain PREROUTING"
        return 1
    fi
    
    return 0
}

config_acc_rule()
{
    local src_ip=$1
    local set_udp_flag=$2
    
    #检查基础规则是否还在
    iptables -t nat -n -L ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ];then
        iptables -t nat -N ${XUNYOU_CHAIN} >/dev/null 2>&1
    fi
    
    iptables -t nat -C PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        iptables -t nat -I PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    fi
    
    iptables -t mangle -n -L ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ];then
        iptables -t mangle -N ${XUNYOU_CHAIN} >/dev/null 2>&1
    fi
    
    iptables -t mangle -C PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        iptables -t mangle -I PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    fi
    
    ip rule add from ${src_ip} fwmark 0x199/0x199 table 199
    ip rule add from ${src_ip} table 198
    
    ip route add default via 172.19.199.1 dev tun199 table 199
    
    iptables -t nat -A ${XUNYOU_CHAIN} -s ${src_ip} -i ${LAN_IF_NAME} -p udp -m udp --dport 53 -j DNAT --to-destination 8.8.8.8 >/dev/null 2>&1 
    
    iptables -t mangle -A ${XUNYOU_CHAIN} -s ${src_ip} -i ${LAN_IF_NAME} -p udp -m udp --dport 53 -j MARK --set-xmark 0x199/0x199 >/dev/null 2>&1 
    
    #主机
    if [ ${set_udp_flag} == "1" ]; then
        iptables -t mangle -I ${XUNYOU_CHAIN} -s ${src_ip} -i ${LAN_IF_NAME} -p udp -m udp --dport 1025:65535 -j MARK --set-xmark 0x199/0x199 >/dev/null 2>&1
        conntrack -D -s ${src_ip} >/dev/null 2>&1
    fi
        
    iptables -t nat -I POSTROUTING -o tun199 -j RETURN >/dev/null 2>&1
    
    iptables -t filter -I INPUT -i tun199 -j ACCEPT >/dev/null 2>&1
    
    iptables -t filter -I FORWARD -i tun199 -j ACCEPT >/dev/null 2>&1
    iptables -t filter -I FORWARD -o tun199 -j ACCEPT >/dev/null 2>&1
}

config_cdn_rule()
{
    local src_ip=$1
    local dst_ip=$2
    local dst_port=$3
    local redirect_port=$4

    iptables -t nat -n -L ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
    if [ $? -ne 0 ];then
        iptables -t nat -N ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
    fi

    iptables -t nat -C ${XUNYOU_CDN_CHAIN} -p tcp --dport ${dst_port} -j REDIRECT --to-ports ${redirect_port} >/dev/null 2>&1
    if [ $? -ne 0 ]; then
        iptables -t nat -A ${XUNYOU_CDN_CHAIN} -p tcp --dport ${dst_port} -j REDIRECT --to-ports ${redirect_port} >/dev/null 2>&1
    fi

    iptables -t nat -I ${XUNYOU_CHAIN} -s ${src_ip} -d ${dst_ip} -p tcp --dport ${dst_port} -j ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
}

clear_acc_rule()
{
    ip route flush table 198 >/dev/null 2>&1
    ip rule delete table 198 >/dev/null 2>&1
    
    ip route flush table 199 >/dev/null 2>&1
    ip rule delete table 199 >/dev/null 2>&1
    
    iptables -t mangle -F ${XUNYOU_CHAIN} >/dev/null 2>&1
    iptables -t nat -F ${XUNYOU_CHAIN} >/dev/null 2>&1
    
    iptables -t nat -F ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
    iptables -t nat -X ${XUNYOU_CDN_CHAIN} >/dev/null 2>&1
    
    iptables -t nat -D POSTROUTING -o tun199 -j RETURN >/dev/null 2>&1
    
    iptables -t filter -D INPUT -i tun199 -j ACCEPT >/dev/null 2>&1
    
    iptables -t filter -D FORWARD -o tun199 -j ACCEPT >/dev/null 2>&1
    iptables -t filter -D FORWARD -i tun199 -j ACCEPT >/dev/null 2>&1
    
    #ip tuntap del mode tun dev tun199 >/dev/null 2>&1

    rm -f ${ACC_CONF}
}

clear_rule()
{
    clear_acc_rule
    iptables -t mangle -D PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    iptables -t mangle -X ${XUNYOU_CHAIN} >/dev/null 2>&1
    
    iptables -t nat -D PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
    iptables -t nat -X ${XUNYOU_CHAIN} >/dev/null 2>&1


    return 0
}

init_rule()
{
    if [ -d ${WORK_DIR} ]; then
        iptables -t mangle -N ${XUNYOU_CHAIN} >/dev/null 2>&1 
        iptables -t mangle -I PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t mangle -F ${XUNYOU_CHAIN} >/dev/null 2>&1
        
        iptables -t nat -N ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t nat -I PREROUTING -i ${LAN_IF_NAME} -j ${XUNYOU_CHAIN} >/dev/null 2>&1
        iptables -t nat -F ${XUNYOU_CHAIN} >/dev/null 2>&1
    fi
    return 0
}

#called by fw3 framework
reload_fw() {
    #将ACC_CONF文件改名，防止清除规则的时候该文件被删除
    if [ -f ${ACC_CONF} ]; then
        mv ${ACC_CONF} ${ACC_CONF}.tmp
    fi

    clear_rule
    init_rule

    if [ -f ${ACC_CONF}.tmp ]; then
        mv ${ACC_CONF}.tmp ${ACC_CONF}

        while read line
        do
            $line
        done < ${ACC_CONF}
    fi
}

#called by plugin-app
close_fw(){
    clear_rule
}

#called by plugin-app
check_fw(){
    check_rule
}

env_init
if [ $? -ne 0 ]; then
    exit 1
fi

case $1 in
    "reload")
        reload_fw
        ;;
    "close")
        close_fw
        ;;
    "status")
        check_fw
        if [ $? -ne 0 ]; then
            #2 is 规则丢失，需要重启进程
            exit 2
        fi
        ;;
    "start_acc")
        config_acc_rule $2 $3
        ;;
    "stop_acc")
        close_fw
        ;;
    "config_cdn")
        config_cdn_rule $2 $3 $4 $5
        ;;
    *)
    #others
    ;;
esac

exit 0